package org.apache.bcel.classfile;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.io.DataInput;
import java.io.DataOutputStream;
import java.io.IOException;
import org.apache.bcel.classfile.Attribute;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.classfile.Utility;
import org.apache.bcel.classfile.Visitor;

public final class Signature extends Attribute {
  private int signatureIndex;
  
  private static boolean identStart(int ch) {
    return (ch == 84 || ch == 76);
  }
  
  public static boolean isActualParameterList(String s) {
    return (s.startsWith("L") && s.endsWith(">;"));
  }
  
  public static boolean isFormalParameterList(String s) {
    return (s.startsWith("<") && s.indexOf(':') > 0);
  }
  
  private static void matchGJIdent(MyByteArrayInputStream in, StringBuilder buf) {
    matchIdent(in, buf);
    int ch = in.read();
    if (ch == 60 || ch == 40) {
      buf.append((char)ch);
      matchGJIdent(in, buf);
      while ((ch = in.read()) != 62 && ch != 41) {
        if (ch == -1)
          throw new IllegalArgumentException("Illegal signature: " + in.getData() + " reaching EOF"); 
        buf.append(", ");
        in.unread();
        matchGJIdent(in, buf);
      } 
      buf.append((char)ch);
    } else {
      in.unread();
    } 
    ch = in.read();
    if (identStart(ch)) {
      in.unread();
      matchGJIdent(in, buf);
    } else if (ch == 41) {
      in.unread();
    } else if (ch != 59) {
      throw new IllegalArgumentException("Illegal signature: " + in.getData() + " read " + (char)ch);
    } 
  }
  
  private static void matchIdent(MyByteArrayInputStream in, StringBuilder buf) {
    int ch;
    if ((ch = in.read()) == -1)
      throw new IllegalArgumentException("Illegal signature: " + in.getData() + " no ident, reaching EOF"); 
    if (!identStart(ch)) {
      StringBuilder stringBuilder = new StringBuilder();
      int count = 1;
      while (Character.isJavaIdentifierPart((char)ch)) {
        stringBuilder.append((char)ch);
        count++;
        ch = in.read();
      } 
      if (ch == 58) {
        int skipExpected = "Ljava/lang/Object".length();
        long skipActual = in.skip(skipExpected);
        if (skipActual != skipExpected)
          throw new IllegalStateException(String.format("Unexpected skip: expected=%,d, actual=%,d", new Object[] { Integer.valueOf(skipExpected), Long.valueOf(skipActual) })); 
        buf.append(stringBuilder);
        ch = in.read();
        in.unread();
      } else {
        for (int i = 0; i < count; i++)
          in.unread(); 
      } 
      return;
    } 
    StringBuilder buf2 = new StringBuilder();
    ch = in.read();
    do {
      buf2.append((char)ch);
      ch = in.read();
    } while (ch != -1 && (Character.isJavaIdentifierPart((char)ch) || ch == 47));
    buf.append(Utility.pathToPackage(buf2.toString()));
    if (ch != -1)
      in.unread(); 
  }
  
  public static String translate(String s) {
    StringBuilder buf = new StringBuilder();
    matchGJIdent(new MyByteArrayInputStream(s), buf);
    return buf.toString();
  }
  
  Signature(int nameIndex, int length, DataInput input, ConstantPool constantPool) throws IOException {
    this(nameIndex, length, input.readUnsignedShort(), constantPool);
  }
  
  public Signature(int nameIndex, int length, int signatureIndex, ConstantPool constantPool) {
    super((byte)10, nameIndex, length, constantPool);
    this.signatureIndex = signatureIndex;
  }
  
  public Signature(org.apache.bcel.classfile.Signature c) {
    this(c.getNameIndex(), c.getLength(), c.getSignatureIndex(), c.getConstantPool());
  }
  
  public void accept(Visitor v) {
    v.visitSignature(this);
  }
  
  public Attribute copy(ConstantPool constantPool) {
    return (Attribute)clone();
  }
  
  public void dump(DataOutputStream file) throws IOException {
    super.dump(file);
    file.writeShort(this.signatureIndex);
  }
  
  public String getSignature() {
    return getConstantPool().getConstantUtf8(this.signatureIndex).getBytes();
  }
  
  public int getSignatureIndex() {
    return this.signatureIndex;
  }
  
  public void setSignatureIndex(int signatureIndex) {
    this.signatureIndex = signatureIndex;
  }
  
  public String toString() {
    String s = getSignature();
    return "Signature: " + s;
  }
}
